public class Calculator
{

  // Operators.
  public static final Operator ADD = new Operator ("+");
  public static final Operator SUBTRACT = new Operator ("-");
  public static final Operator MULTIPLY = new Operator ("*");
  public static final Operator DIVIDE = new Operator ("/");
  public static final Operator MOVE = new Operator (" ");

  // Values are *entered* (button pushes) into the display variable.
  // The value in display is presented onscreen in the calculator.
  // Results are computed and stored in the accumulator.
  private double display, accumulator;

  // The operation to apply when updating the accumulator.
  private Operator pendingOperation;

  // Do we clear the display variable before entering digits?
  private boolean clearEntryPending;

  // When a digit is pressed is it before the decimal point?
  private boolean beforeDecimal;

  // Position for next digit after the decimal point
  private double decimalPlace;

  public Calculator ()
  {
    clear ();
  }

  // Called from the pressed() method in ClearButton and
  // from constructor of this class.
  public void clear ()
  {
    accumulator = 0.0;
    pendingOperation = MOVE;
    clearEntry ();
  }

  // Called from the pressed() method in ClearEntryButton
  // (when "CE" button pressed), and the following methods
  // in this class: clear(), this.digit(), this.decimal().
  // For the latter two, this method is only called if
  // clearEntryPending is true.
  public void clearEntry ()
  {
    display = 0.0;
    clearEntryPending = false;
    beforeDecimal = true;
    decimalPlace = 0.1;
  }

  // Called from pressed() method in any of the 10
  // DigitButton objects (with labels "0" through "9").
  public void digit (int digit) 
  {
    assert digit >= 0 && digit <= 9;
    if (clearEntryPending) {
      clearEntry ();
    }
    if (beforeDecimal) {
      display = 10*display + digit;
    } else {
      display += decimalPlace*digit;
      decimalPlace *= 0.1;
    }
  }

  // Called from pressed() in DecimalButton when
  // "." pressed.
  public void decimal () 
  {
    if (clearEntryPending) {
      clearEntry ();
    }
    beforeDecimal = false;
  }

  // Called from pressed() method in OperatorButton for
  // "+", "-", "*", and "/".
  public void operator (Operator operator) 
  {
    equals ();
    pendingOperation = operator;
  }

  // Called from pressed() method in EqualsButton and
  // operator() method in this class.
  public void equals () 
  {
    if (pendingOperation == ADD) {
      accumulator += display;
    } else if (pendingOperation == SUBTRACT) {
      accumulator -= display;
    } else if (pendingOperation == MULTIPLY) {
      accumulator *= display;
    } else if (pendingOperation== DIVIDE) {
      accumulator /= display;
    } else {
      accumulator = display;
    }
    display = accumulator;
    clearEntryPending = true;
    pendingOperation = MOVE;
  }

  // Called in actionPerformed() in CalculatorButton.
  public String toString ()
  {
    return Double.toString (display);
  }

  public static void main (String [] args)
  {
    // Simulate "10 + 2 =" without GUI.
    Calculator c = new Calculator ();
    c.digit (1);
    System.out.println ("1 = " + c);
    c.digit (0);
    System.out.println ("10 = " +  c);
    c.operator (ADD);
    System.out.println ("10 = " +  c);
    c.digit (2);
    System.out.println ("2 = " +  c);
    c.equals ();
    System.out.println ("12 = " +  c);
  }
}
